using System;
using System.Data;
using System.Collections;
using System.Threading;
using System.Security.Principal;
using System.Web.UI.WebControls;

using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.EventLog;
using CMS.UIControls;

using TreeNode = CMS.TreeEngine.TreeNode;

public partial class CMSDesk_Content_delete : CMSDeskPage
{
    #region "Private variables"

    protected string[] nodeIDs = null;
    protected int cancelNodeId = 0;
    protected CurrentUserInfo currentUser = null;
    protected SiteInfo currentSite = null;
    protected static Hashtable mLogs = new Hashtable();
    protected static Hashtable mErrors = new Hashtable();
    protected static Hashtable mInfos = new Hashtable();
    protected static string refreshScript;

    #endregion


    #region "Properties"

    /// <summary>
    /// Current log
    /// </summary>
    public string CurrentLog
    {
        get
        {
            return ValidationHelper.GetString(mLogs["DeleteLog_" + ctlAsync.ProcessGUID], string.Empty);
        }
        set
        {
            mLogs["DeleteLog_" + ctlAsync.ProcessGUID] = value;
        }
    }


    /// <summary>
    /// Current Error
    /// </summary>
    public string CurrentError
    {
        get
        {
            return ValidationHelper.GetString(mErrors["DeleteError_" + ctlAsync.ProcessGUID], string.Empty);
        }
        set
        {
            mErrors["DeleteError_" + ctlAsync.ProcessGUID] = value;
        }
    }


    /// <summary>
    /// Current Info
    /// </summary>
    public string CurrentInfo
    {
        get
        {
            return ValidationHelper.GetString(mInfos["DeleteInfo_" + ctlAsync.ProcessGUID], string.Empty);
        }
        set
        {
            mInfos["DeleteInfo_" + ctlAsync.ProcessGUID] = value;
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        // Initialize events
        ctlAsync.OnFinished += ctlAsync_OnFinished;
        ctlAsync.OnError += ctlAsync_OnError;
        ctlAsync.OnRequestLog += ctlAsync_OnRequestLog;
        ctlAsync.OnCancel += ctlAsync_OnCancel;
        // Current Node ID to delete
        if (Request.QueryString["nodeid"] != null)
        {
            nodeIDs = Request.QueryString["nodeid"].Trim(';').Split(';');
        }

        // Initialize current site
        if (CMSContext.CurrentSite != null && currentSite == null)
        {
            currentSite = CMSContext.CurrentSite;
        }
        // Initialize current user
        if (CMSContext.CurrentUser != null && currentUser == null)
        {
            currentUser = CMSContext.CurrentUser;
        }

        if (!IsCallback)
        {
            // Initialize resource strings, images
            string imagesUrl = "~/App_Themes/" + Theme + "/Images/CMSDesk/Content/";

            // Setup page title text and image
            CurrentMaster.Title.TitleText = ResHelper.GetString("Content.DeleteTitle");
            CurrentMaster.Title.TitleImage = imagesUrl + "delete.gif";

            btnCancel.Attributes.Add("onclick", "CancelAction(); return false;");

            // Register the dialog script
            Page.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY,
                                                             ScriptHelper.DialogScript);

            titleElemAsync.TitleText = ResHelper.GetString("ContentDelete.DeletingDocuments");
            titleElemAsync.TitleImage = imagesUrl + "delete.gif";

            // Set visibility of panels
            pnlContent.Visible = true;
            pnlLog.Visible = false;

            // Set all cultures checkbox
            DataSet culturesDS = CultureInfoProvider.GetSiteCultures(currentSite.SiteName);
            if ((DataHelper.DataSourceIsEmpty(culturesDS)) || (culturesDS.Tables[0].Rows.Count <= 1))
            {
                chkAllCultures.Checked = true;
            }

            if (nodeIDs != null)
            {
                if (nodeIDs.Length == 1)
                {
                    int nodeId = ValidationHelper.GetInteger(nodeIDs[0], 0);
                    // Get the node name
                    TreeProvider tree = new TreeProvider(currentUser);

                    // Get any culture if current not found
                    TreeNode node = tree.SelectSingleNode(nodeId, currentUser.PreferredCultureCode) ?? tree.SelectSingleNode(nodeId, TreeProvider.ALL_CULTURES);
                    if (node != null)
                    {
                        if (!IsUserAuthorizedToDeleteDocument(node, false, false, CMSContext.CurrentSiteName))
                        {
                            pnlDelete.Visible = false;
                            lblError.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtodeletedocument"), node.NodeAliasPath);
                        }

                        if (node.IsLink)
                        {
                            CurrentMaster.Title.TitleText = ResHelper.GetString("Content.DeleteTitleLink") + " \"" + HTMLHelper.HTMLEncode(node.DocumentName) + "\"";
                            lblQuestion.Text = ResHelper.GetString("ContentDelete.QuestionLink");
                            chkAllCultures.Checked = true;
                            plcCheck.Visible = false;
                        }
                        else
                        {
                            CurrentMaster.Title.TitleText = ResHelper.GetString("Content.DeleteTitle") + " \"" + HTMLHelper.HTMLEncode(node.DocumentName) + "\"";

                            // If there is SKU
                            if ((currentUser.IsAuthorizedPerResource("CMS.Ecommerce", "EcommerceModify")) && (node.HasSKU))
                            {
                                pnlDeleteSKU.Visible = true;
                                chkDeleteSKU.Visible = true;
                            }
                        }

                        // Show/Hide checkboxes                
                        if (CanDestroy(node))
                        {
                            chkDestroy.Visible = true;
                            chkAllCultures.Visible = !chkAllCultures.Checked;
                        }

                        cancelNodeId = node.NodeID;
                    }
                }
                else if (nodeIDs.Length > 1)
                {
                    TreeProvider tree = new TreeProvider(currentUser);
                    string where = "NodeID IN (";
                    foreach (string nodeID in nodeIDs)
                    {
                        where += nodeID.Replace("'", "''") + ",";
                    }
                    where = where.TrimEnd(',') + ")";

                    TreeNode node = null;

                    DataSet ds = tree.SelectNodes(currentSite.SiteName, "/%", TreeProvider.ALL_CULTURES, true, null, where, null, -1, false);


                    if (!DataHelper.DataSourceIsEmpty(ds))
                    {
                        foreach (DataRow dr in ds.Tables[0].Rows)
                        {
                            cancelNodeId = ValidationHelper.GetInteger(dr["NodeParentID"], 0);
                            bool isLink = (dr["NodeLinkedNodeID"] != DBNull.Value);
                            string name = (string)dr["DocumentName"];
                            if (isLink)
                            {
                                name += "*";
                            }
                            lblDocuments.Text += HTMLHelper.HTMLEncode(name) + "<br />";

                            // Set visibility of checkboxes
                            node = new TreeNode(dr, ValidationHelper.GetString(dr["ClassName"], ""));

                            if (!IsUserAuthorizedToDeleteDocument(node, false, false, CMSContext.CurrentSiteName))
                            {
                                pnlDelete.Visible = false;
                                lblError.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtodeletedocument"), node.NodeAliasPath);
                                break;
                            }

                            if (CanDestroy(node))
                            {
                                chkDestroy.Visible = true;
                                chkAllCultures.Visible = !chkAllCultures.Checked;
                            }
                            if ((currentUser.IsAuthorizedPerResource("CMS.Ecommerce", "EcommerceModify")) && (node.HasSKU))
                            {
                                pnlDeleteSKU.Visible = true;
                                chkDeleteSKU.Visible = true;
                            }
                        }
                    }

                    lblQuestion.Text = ResHelper.GetString("ContentDelete.QuestionMultiple") + "<br />";
                    CurrentMaster.Title.TitleText = ResHelper.GetString("Content.DeleteTitleMultiple");
                    chkAllCultures.Text = ResHelper.GetString("ContentDelete.AllCulturesMultiple");
                    chkDestroy.Text = ResHelper.GetString("ContentDelete.DestroyMultiple");
                    chkDeleteSKU.Text = ResHelper.GetString("ContentDelete.SKUMultiple");
                }
            }
        }
    }


    protected override void OnPreInit(EventArgs e)
    {
        ((Panel)CurrentMaster.PanelBody.FindControl("pnlContent")).CssClass = string.Empty;
        base.OnPreInit(e);
    }


    protected override void OnPreRender(EventArgs e)
    {
        lblError.Visible = (lblError.Text != string.Empty);
        lblInfo.Visible = (lblInfo.Text != string.Empty);
        base.OnPreRender(e);
    }

    #endregion


    #region "Button actions"

    protected void btnNo_Click(object sender, EventArgs e)
    {
        AddScript("DisplayDocument(" + cancelNodeId + ");");
    }


    protected void btnOK_Click(object sender, EventArgs e)
    {
        pnlLog.Visible = true;
        pnlContent.Visible = false;

        CurrentLog = string.Empty;
        CurrentError = string.Empty;
        CurrentInfo = string.Empty;

        ctlAsync.Parameter = currentUser.PreferredCultureCode + ";" + CMSContext.CurrentSiteName;
        ctlAsync.Worker.RunAsync(Delete, WindowsIdentity.GetCurrent());
    }

    #endregion


    #region "Private methods"

    /// <summary>
    /// Deletes document(s)
    /// </summary>
    private void Delete(object parameter)
    {
        if (parameter == null)
        {
            return;
        }

        string[] parameters = ((string)parameter).Split(new char[] { ';' });

        string cultureCode = parameters[0];
        string siteName = parameters[1];

        // Prepare the where condition
        string where = "NodeID IN (";
        foreach (string id in nodeIDs)
        {
            where += id.Replace("'", "''") + ",";
        }
        where = where.TrimEnd(',') + ")";

        // Get the documents
        TreeProvider tree = new TreeProvider(currentUser);
        DataSet ds = null;

        if (chkAllCultures.Checked)
        {
            ds = tree.SelectNodes(siteName, "/%", TreeProvider.ALL_CULTURES, true, null, where, null, -1, false);
        }
        else
        {
            ds = tree.SelectNodes(siteName, "/%", cultureCode, false, null, where, null, -1, false);
        }

        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            // Begin log
            AddLog(ResHelper.GetString("ContentDelete.DeletingDocuments"));

            // Delete the documents
            foreach (DataRow dr in ds.Tables[0].Rows)
            {
                string aliasPath = ValidationHelper.GetString(dr["NodeAliasPath"], string.Empty);
                string culture = ValidationHelper.GetString(dr["DocumentCulture"], string.Empty);
                string className = ValidationHelper.GetString(dr["ClassName"], string.Empty);

                // Get the node
                TreeNode node = tree.SelectSingleNode(siteName, aliasPath, culture, false, className, false);

                if (node != null)
                {
                    int parentId = node.NodeParentID;
                    // Check delete permissions
                    if (IsUserAuthorizedToDeleteDocument(node, chkDestroy.Checked, chkAllCultures.Checked, siteName))
                    {
                        // Delete the document
                        try
                        {
                            // Log alias path being deleted
                            AddLog(node.NodeAliasPath);
                            if (parentId <= 0)
                            {
                                parentId = node.NodeID;
                            }

                            // Delete document
                            if (DocumentHelper.DeleteDocument(node, tree, chkAllCultures.Checked, chkDestroy.Checked, chkDeleteSKU.Checked))
                            {
                                // Refresh tree
                                refreshScript = "RefreshNode(" + parentId + ", " + parentId + "); \n" +
                                                "DisplayDocument(" + parentId + ");";
                            }
                            else
                            {
                                refreshScript = "RefreshNode(" + node.NodeID + ", " + node.NodeID + "); \n" +
                                                "DisplayDocument(" + node.NodeID + ");";
                            }
                        }
                        catch (ThreadAbortException)
                        {
                            // When canceled
                            refreshScript = "";
                            CurrentInfo = ResHelper.GetString("DeleteDocument.DeletionCanceled");
                            AddLog(CurrentInfo);
                            return;
                        }
                        catch (Exception ex)
                        {
                            // Log error
                            refreshScript = "";
                            CurrentError = ResHelper.GetString("ContentRequest.DeleteFailed") + ": " + ex.Message;
                            AddLog(CurrentError);
                            EventLogProvider log = new EventLogProvider(tree.Connection);
                            log.LogEvent("E", DateTime.Now, "Content", "DELETEDOC", currentUser.UserID,
                                         currentUser.UserName, node.NodeID, node.DocumentName,
                                         HTTPHelper.GetUserHostAddress(), ex.Message + ": " + ex.StackTrace,
                                         currentSite.SiteID, HTTPHelper.GetAbsoluteUri());
                            return;
                        }
                    }
                    // Access denied - not authorized to delete the document
                    else
                    {
                        refreshScript = string.Empty;
                        CurrentError = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtodeletedocument"), node.NodeAliasPath);
                        AddLog(CurrentError);
                        return;
                    }
                }
                else
                {
                    refreshScript = string.Empty;
                    CurrentError = ResHelper.GetString("ContentRequest.ErrorMissingSource");
                    AddLog(CurrentError);
                    return;
                }
            }
        }
        else
        {
            refreshScript = "";
            CurrentError = ResHelper.GetString("DeleteDocument.CultureNotExists");
            AddLog(CurrentError);
            return;
        }
    }

    #endregion


    #region "Help methods"

    /// <summary>
    /// Adds the script to the output request window
    /// </summary>
    /// <param name="script">Script to add</param>
    public override void AddScript(string script)
    {
        ltlScript.Text += ScriptHelper.GetScript(script);
    }


    /// <summary>
    /// Indicates whether specified node can be destroyed by current user.
    /// </summary>
    /// <param name="node">Tree node to check.</param>
    private bool CanDestroy(TreeNode node)
    {
        return (currentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Destroy) == AuthorizationResultEnum.Allowed);
    }


    /// <summary>
    /// Checks whether the user is authorized to delete document.
    /// </summary>
    /// <param name="node">Document node</param>
    /// <param name="deleteDocHistory">Destroy document history?</param>
    /// <param name="deleteAllCultureVersions">Delete all culture versions?</param>
    /// <param name="siteName">Site name</param>
    protected bool IsUserAuthorizedToDeleteDocument(TreeNode node, bool deleteDocHistory, bool deleteAllCultureVersions, string siteName)
    {
        bool isAuthorized = true;
        // Check delete permission
        if (currentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Delete, siteName) == AuthorizationResultEnum.Allowed)
        {
            if (deleteDocHistory)
            {
                // Check destroy permisson
                if (!(currentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Destroy, siteName) == AuthorizationResultEnum.Allowed))
                {
                    isAuthorized = false;
                }
            }
        }
        else
        {
            isAuthorized = false;
        }

        return isAuthorized;
    }

    #endregion


    #region "Handling async thread"

    private void ctlAsync_OnCancel(object sender, EventArgs e)
    {
        ltlScript.Text += ScriptHelper.GetScript("var __pendingCallbacks = new Array();");
        lblError.Text = CurrentError;
        lblInfo.Text = CurrentInfo;
    }


    private void ctlAsync_OnRequestLog(object sender, EventArgs e)
    {
        ctlAsync.Log = CurrentLog;
    }


    private void ctlAsync_OnError(object sender, EventArgs e)
    {
        lblError.Text = CurrentError;
        lblInfo.Text = CurrentInfo;
    }


    private void ctlAsync_OnFinished(object sender, EventArgs e)
    {
        lblError.Text = CurrentError;
        lblInfo.Text = CurrentInfo;
        // Set right position in menu
        AddScript(refreshScript);
    }


    /// <summary>
    /// Adds the log information
    /// </summary>
    /// <param name="newLog">New log information</param>
    protected void AddLog(string newLog)
    {
        CurrentLog = (newLog + "<br />" + CurrentLog);
    }

    #endregion
}
