using System;

using CMS.SettingsProvider;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.PortalEngine;
using CMS.UIControls;
using CMS.WorkflowEngine;
using CMS.FileManager;

public partial class CMSDesk_Content_editpage : CMSDeskPage
{
    protected string viewpage = "../blank.htm";
    protected TreeNode node = null;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Get the node ID
        int nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);

        viewpage = "edit.aspx" + Request.Url.Query;

        string action = ValidationHelper.GetString(Request.QueryString["action"], "").ToLower();

        switch (action)
        {
            case "newlink":
                viewpage = "NewLink.aspx" + Request.Url.Query;
                break;

            default:
                // Check if new document desired
                bool newdocument = (action == "new");
                DataClassInfo classInfo = null;
                if (newdocument)
                {
                    // Get the class ID
                    int classId = ValidationHelper.GetInteger(Request.QueryString["classid"], 0);
                    classInfo = DataClassInfoProvider.GetDataClass(classId);
                }
                else
                {
                    // Get the document
                    TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
                    node = tree.SelectSingleNode(nodeId);
                    if (node != null)
                    {
                        classInfo = DataClassInfoProvider.GetDataClass(node.NodeClassName);
                    }
                }

                // Check the editing page change
                if (classInfo != null)
                {
                    if (CMSContext.ViewMode == ViewModeEnum.EditForm || newdocument)
                    {
                        // If new document, check if new page URL is set
                        if (newdocument)
                        {
                            if (!string.IsNullOrEmpty(classInfo.ClassNewPageURL))
                            {
                                viewpage = ResolveUrl(classInfo.ClassNewPageURL) + Request.Url.Query;
                            }
                        }
                        // If existing document, check if editing page URL is set
                        else if (!string.IsNullOrEmpty(classInfo.ClassEditingPageURL))
                        {
                            viewpage = ResolveUrl(classInfo.ClassEditingPageURL) + Request.Url.Query;
                        }
                    }
                    else if (CMSContext.ViewMode == ViewModeEnum.Edit)
                    {
                        // Check if view page URL is set
                        if (!string.IsNullOrEmpty(classInfo.ClassViewPageUrl))
                        {
                            viewpage = ResolveUrl(classInfo.ClassViewPageUrl) + Request.Url.Query;
                        }
                        else
                        {
                            viewpage = ResolveUrl(TreePathUtils.GetPermanentDocUrl(node.NodeGUID, node.NodeAlias, CMSContext.CurrentSiteName, "cmsgetdoc"));
                        }
                    }
                    else if (CMSContext.ViewMode == ViewModeEnum.Design)
                    {
                        // Use permanent URL to get proper design mode
                        viewpage = ResolveUrl(TreePathUtils.GetPermanentDocUrl(node.NodeGUID, node.NodeAlias, CMSContext.CurrentSiteName, "cmsgetdoc"));
                    }
                    else
                    {
                        // Use standard URL to get other modes
                        viewpage = ResolveUrl(CMSContext.GetUrl(node.NodeAliasPath, node.DocumentUrlPath));
                    }
                }
                break;
        }

        pageview.Attributes.Add("src", viewpage);
    }
}
